#pragma once
/* -*-c++-*- osgVisual - Copyright (C) 2009-2011 Torben Dannhauer
 *
 * This library is based on OpenSceneGraph, open source and may be redistributed and/or modified under 
 * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or
 * (at your option) any later version.  The full license is in LICENSE file
 * included with this distribution, and on the openscenegraph.org website.
 *
 * osgVisual requires for some proprietary modules a license from the correspondig manufacturer.
 * You have to aquire licenses for all used proprietary modules.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * OpenSceneGraph Public License for more details.
*/

#include <region.h>
#include "myTileLoadedCallback.h"
#include "terrainModificationTechnique.h"
#include <osgTerrain/Terrain>
#include <osgTerrain/GeometryTechnique>
#include <osg/observer_ptr>
#include <vector>

#include "Export.h"



namespace osgTerrain {


/**
 * \brief This class provides a management interface to control terrain modification.
 * 
 * It is possible to modify preprocessed VirtualPlanetBuilder (VPB) Databases on the fly.
 * The modification can be performed in two ways: 
 * a) Modifying only the tile data and using the standard geometryTechnique to mesh the drawable.
 * b) Using an user defined geometryTechnique to create other meshes than the regular triangle mesh.
 * 
 * Currently the changes are made on run time but in the pseudo loader. This way the modification does not affect the frame rate but only the delay in the tile delivery to the rendering thread.
 * 
 * @author Torben Dannhauer
 * @date  Mrz 2011
 */ 
class TERRAINMODMANAGER_EXPORT ModificationManager
{
public:
	//! Destructor: Must be public to ensure the singleton can be cleand up at the end of the programm.
	~ModificationManager(){};

	/**
	 * \brief This function returns a reference to the singleton instance of the terrain manager.
	 * 
	 * @return Reference to the singleton instance of the terrain manager.
	 */ 
	static ModificationManager* getInstance() {static ModificationManager instance; return &instance;}

	/**
	 * \brief This function adds a terrain class to the manager to be managed and accessible to terrain modifications.
	 * 
	 * @param terrain : Terrain to manage.
	 */ 
	void addTerrainToManage(osgTerrain::Terrain* terrain);

	/**
	 * \brief This functions removes a terrain class from the manager to stop it's managing.
	 * 
	 * @param terrain : Terrain to remove.
	 * @return : True of the specified terrain was managed and is now removed. If the specified terrain was not managed it will return false.
	 */ 
	bool removeTerrainToManage(osgTerrain::Terrain* terrain);



	//void addRoi(region modificationROI, terrainModificationTechnique* technique=NULL);
	//bool removeRoi(region modificationROI);



	/**
	 * \brief This functions sets the geometryTechniqueProtoype. The geometryTechnique is responsible for creating a renderable mesh from the tile data.
	 * 
	 * If you want to use a geometryTechnique different from the standard one, you have to use this function.
	 * 
	 * @param geomTechnique : GeometryTechnique you want to use for the specified terrain.
	 * @param terrain : Terrain you want to set it's geometryTechniquePrototype. If NULL, the geometrytechnique will be applied to all terrains.
	 */ 
	void setGeometryTechniquePrototype( osgTerrain::GeometryTechnique* geomTechnique, osgTerrain::Terrain* terrain=NULL );

	/**
	 * \brief This function sets the terrainModificationTechnique.
	 * 
	 * The terrainModificationTechnique can modify the tile data before the data is used by the geometryTechnique to create the renderable mesh.
	 * 
	 * @param terrainModTechnique 
	 * @param terrain 
	 */ 
	void setTerrainModificationTechnique( osgTerrain::GeometryTechnique* terrainModTechnique, osgTerrain::Terrain* terrain=NULL );


private:
	//! Constructor: Private to avoid that this class can be instantiated - it should only be usable as singleton.
	ModificationManager(){};

	//! Copy-Constructor: Private to avoid that someone can create an class instance by copying the singleton instance.
	ModificationManager(const ModificationManager& cc) {cc;};

	//! Vector of all managed terrain pointers.
	std::vector<osgTerrain::Terrain*> managedTerrain;

};

}	// Namespace end.